# -*- coding: utf-8 -*-

"""
Meetlify: Static Site Generator for Meetup Websites
A Python Package for Generating Static Website for Meetups.
https://github.com/pybodensee/meetlify

    src\meetlify\page.py

    Copyright (C) 2024-2024 Faisal Shahzad <info@serpwings.com>

<LICENSE_BLOCK>
Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
</LICENSE_BLOCK>
"""

# +++++++++++++++++++++++++++++++++++++++++++++++++++++
# STANDARD LIBARY IMPORTS
# +++++++++++++++++++++++++++++++++++++++++++++++++++++

from pathlib import Path
from dataclasses import dataclass
from datetime import datetime, timezone

# +++++++++++++++++++++++++++++++++++++++++++++++++++++
# 3rd PARTY LIBRARY IMPORTS
# +++++++++++++++++++++++++++++++++++++++++++++++++++++

from slugify import slugify

# +++++++++++++++++++++++++++++++++++++++++++++++++++++
# INTERNAL IMPORTS
# +++++++++++++++++++++++++++++++++++++++++++++++++++++

from .utils import markdown_convertor
from .constants import STATUS

# +++++++++++++++++++++++++++++++++++++++++++++++++++++
# IMPLEMENATIONS
# +++++++++++++++++++++++++++++++++++++++++++++++++++++


@dataclass
class Page:
    title: str
    author: str
    description: str
    slug: str
    create_date: datetime
    last_modified: datetime
    toc: str
    content: str
    add_to_sitemap: bool
    status: str

    def __lt__(self, other_):
        return self.create_date < other_.create_date

    @classmethod
    def from_markdown(cls, page_md_: Path):
        meta, toc, content = markdown_convertor(page_md_)
        return cls(
            title=meta.get("title"),
            author=meta.get("author"),
            description=meta.get("description"),
            slug=meta.get("slug") or slugify(meta.get("title")),
            create_date=datetime.strptime(meta.get("create_date"), "%Y-%m-%d::%H:%M"),
            last_modified=datetime.fromtimestamp(
                page_md_.stat().st_mtime, tz=timezone.utc
            ),
            toc=toc,
            content=content,
            add_to_sitemap=bool(meta.get("add_to_sitemap")),
            status=meta.get("status"),
        )


class Pages:
    def __init__(self, *, path_: Path, reverse_: bool = True) -> None:
        self.content = sorted(
            [
                Page.from_markdown(page_md)
                for page_md in path_.iterdir()
                if page_md.is_file() and page_md.suffix == ".md"
            ],
            reverse=reverse_,
        )

    def __getitem__(self, status_: list[STATUS] | STATUS) -> list[Page]:
        if isinstance(status_, STATUS):
            status_ = [status_]
        return [
            content
            for content in self.content
            if content.status in [stat.value for stat in status_]
        ]



# -*- coding: utf-8 -*-

"""
Meetlify: Static Site Generator for Meetup Websites
A Python Package for Generating Static Website for Meetups.
https://github.com/pybodensee/meetlify

    setup.py
    
    Copyright (C) 2024-2024 Faisal Shahzad <info@serpwings.com>

<LICENSE_BLOCK>
Permission is hereby granted, free of charge, to any person obtaining a copy of 
this software and associated documentation files (the "Software"), to deal in 
the Software without restriction, including without limitation the rights to 
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
of the Software, and to permit persons to whom the Software is furnished to do 
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all 
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
SOFTWARE.
</LICENSE_BLOCK>
"""


# +++++++++++++++++++++++++++++++++++++++++++++++++++++
# STANDARD LIBARY IMPORTS
# +++++++++++++++++++++++++++++++++++++++++++++++++++++

import os
from setuptools import (
    find_packages,
    setup,
)

# +++++++++++++++++++++++++++++++++++++++++++++++++++++
# INTERNAL IMPORTS
# +++++++++++++++++++++++++++++++++++++++++++++++++++++

from src.meetlify import __version__

# +++++++++++++++++++++++++++++++++++++++++++++++++++++
# DATABASE/CONSTANTS LIST
# +++++++++++++++++++++++++++++++++++++++++++++++++++++

python_minor_min = 11
python_minor_max = 12
confirmed_python_versions = [
    "Programming Language :: Python :: 3.{MINOR:d}".format(MINOR=minor)
    for minor in range(python_minor_min, python_minor_max + 1)
]

# Fetch readme file
with open(os.path.join(os.path.dirname(__file__), "README.md")) as f:
    long_description = f.read()

# Define source directory (path)
SRC_DIR = "src"

# Requirements for dev and gui
extras_require = {
    "dev": [
        "black",
        "python-language-server[all]",
        "setuptools",
        "twine",
        "wheel",
        "setuptools",
        "pytest",
        "pytest-cov",
        "twine",
        "wheel",
        "mkdocs",
        "mkdocs-gen-files",
        "mkdocstrings[python]",
        "pymdown-extensions",
    ],
}
extras_require["all"] = list(
    {rq for target in extras_require.keys() for rq in extras_require[target]}
)

# Install package
setup(
    name="meetlify",
    packages=find_packages(SRC_DIR),
    package_dir={"": SRC_DIR},
    version=__version__,
    description="Python Package to Generate Meetup Websites",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Faisal Shahzad",
    author_email="pybodensee@gmail.com",
    url="https://github.com/pybodensee/meetlify",
    download_url="https://github.com/pybodensee/meetlify/releases/v%s.tar.gz"
    % __version__,
    license="MIT",
    keywords=["meetups", "static-site-generators", "seo", "python"],
    scripts=[],
    include_package_data=True,
    python_requires=">=3.{MINOR:d}".format(MINOR=python_minor_min),
    setup_requires=[],
    install_requires=["click", "markdown", "Jinja2", "python-slugify"],
    extras_require=extras_require,
    zip_safe=False,
    entry_points={
        "console_scripts": [
            "meetlify = meetlify.cli:main",
        ],
    },
    classifiers=[
        "Development Status :: 1 - Planning",
        "Environment :: Console",
        "Environment :: X11 Applications",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Science/Research",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS",
        "Operating System :: POSIX :: BSD",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3",
    ]
    + confirmed_python_versions
    + [
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Scientific/Engineering",
        "Topic :: System",
        "Topic :: System :: Archiving",
        "Topic :: System :: Archiving :: Backup",
        "Topic :: System :: Archiving :: Mirroring",
        "Topic :: System :: Filesystems",
        "Topic :: System :: Systems Administration",
        "Topic :: Utilities",
    ],
)



# -*- coding: utf-8 -*-

"""
Meetlify: Static Site Generator for Meetup Websites
A Python Package for Generating Static Website for Meetups.
https://github.com/pybodensee/meetlify

    src\meetlify\configs.py

    Copyright (C) 2024-2024 Faisal Shahzad <info@serpwings.com>

<LICENSE_BLOCK>
Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
</LICENSE_BLOCK>
"""


# +++++++++++++++++++++++++++++++++++++++++++++++++++++
# STANDARD LIBARY IMPORTS
# +++++++++++++++++++++++++++++++++++++++++++++++++++++

import json
import codecs
from pathlib import Path
from dataclasses import dataclass


# +++++++++++++++++++++++++++++++++++++++++++++++++++++
# IMPLEMENATIONS
# +++++++++++++++++++++++++++++++++++++++++++++++++++++


@dataclass
class Banner:
    """Banner Data Class to hold Banners on Posts"""

    name: str
    type_: str
    message: str


@dataclass
class Menu:
    """Menu Data Class to hold Header and footer menus"""

    header: dict
    footer: dict


@dataclass
class Folders:
    """Folder data class for differnt types of folder used in Processing"""

    output: str
    themes: str
    images: str
    content: str
    meetups: str
    pages: str
    posts: str
    categories: str


@dataclass
class Configs:
    """Config Data Class to hold Project Configurations"""

    name: str
    URL: str
    language: str
    theme: str
    title: str
    author: str
    email: str
    description: str
    sitemap: bool
    feeds: bool
    robots: bool
    logo: str
    favicon: str
    copyright: str
    home: str
    folders: dict
    menu: Menu
    about_us: list[str]
    banners: list[Banner]

    @classmethod
    def from_json(cls, json_file_: Path):
        assert isinstance(json_file_, Path)
        assert json_file_.exists()

        with codecs.open(json_file_, "r", encoding="utf-8") as f:
            cfgs = json.load(f)
            return cls(
                name=cfgs.get("name"),
                URL=cfgs.get("URL"),
                language=cfgs.get("language"),
                theme=cfgs.get("theme"),
                title=cfgs.get("title"),
                author=cfgs.get("author"),
                email=cfgs.get("email"),
                description=cfgs.get("description"),
                sitemap=cfgs.get("sitemap"),
                feeds=cfgs.get("feeds"),
                robots=cfgs.get("robots"),
                logo=cfgs.get("logo"),
                favicon=cfgs.get("favicon"),
                copyright=cfgs.get("copyright"),
                home=cfgs.get("home"),
                folders=Folders(**cfgs.get("folders")),
                menu=Menu(**cfgs.get("menu")),
                about_us=cfgs.get("about_us"),
                banners=[Banner(**banner) for banner in cfgs.get("banners")],
            )

    def get_banner(self, banner_name: str) -> Banner:

        return next(
            (banner for banner in self.banners if banner.name == banner_name),
            Banner(name=None, type_="", message=""),
        )
